@@ -1,7 +1,7 @@
 # vim: ts=8:sts=8:sw=8:noexpandtab
 
 # This file is part of ReText
-# Copyright: 2012-2014 Dmitry Shachnev
+# Copyright: 2012-2017 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ from os.path import dirname, exists, join
 from PyQt5.QtCore import QByteArray, QLocale, QSettings, QStandardPaths
 from PyQt5.QtGui import QFont
 
-app_version = "6.0.2"
+app_version = "7.0.1"
 
 settings = QSettings('ReText project', 'ReText')
 
@@ -33,36 +33,52 @@ if not str(settings.fileName()).endswith('.conf'):
 	settings = QSettings(QSettings.IniFormat, QSettings.UserScope,
 		'ReText project', 'ReText')
 
-datadirs = QStandardPaths.standardLocations(QStandardPaths.GenericDataLocation)
-datadirs = [join(d, 'retext') for d in datadirs]
+datadirs = []
 
-if sys.platform == "win32":
-	# Windows compatibility: Add "PythonXXX\share\" path
-	datadirs.append(join(dirname(sys.executable), 'share', 'retext'))
+def initializeDataDirs():
+	assert not datadirs
 
-if '__file__' in locals():
-	datadirs = [dirname(dirname(__file__))] + datadirs
+	if '__file__' in locals():
+		datadirs.append(dirname(dirname(__file__)))
 
-icon_path = 'icons/'
-for dir in datadirs:
-	if exists(join(dir, 'icons')):
-		icon_path = join(dir, 'icons/')
-		break
+	dataLocations = QStandardPaths.standardLocations(QStandardPaths.GenericDataLocation)
+	datadirs.extend(join(d, 'retext') for d in dataLocations)
+
+	if sys.platform == "win32":
+		# Windows compatibility: Add "PythonXXX\share\" path
+		datadirs.append(join(dirname(sys.executable), 'share', 'retext'))
+
+	# For virtualenvs
+	datadirs.append(join(dirname(dirname(sys.executable)), 'share', 'retext'))
+
+_iconPath = None
+
+def getBundledIcon(iconName):
+	global _iconPath
+	if _iconPath is None:
+		for dir in ['icons'] + datadirs:
+			_iconPath = join(dir, 'icons')
+			if exists(_iconPath):
+				break
+	return join(_iconPath, iconName + '.png')
 
 configOptions = {
 	'appStyleSheet': '',
 	'autoSave': False,
 	'defaultCodec': '',
 	'defaultMarkup': markups.MarkdownMarkup.name,
+	'detectEncoding': True,
 	'editorFont': QFont(),
 	'font': QFont(),
 	'handleWebLinks': False,
 	'hideToolBar': False,
 	'highlightCurrentLine': False,
 	'iconTheme': '',
+	'lastTabIndex': 0,
 	'lineNumbersEnabled': False,
 	'livePreviewByDefault': False,
 	'markdownDefaultFileExtension': '.mkd',
+	'openLastFilesOnStartup': False,
 	'pygmentsStyle': 'default',
 	'restDefaultFileExtension': '.rst',
 	'rightMargin': 0,
@@ -71,10 +87,12 @@ configOptions = {
 	'spellCheckLocale': '',
 	'styleSheet': '',
 	'syncScroll': True,
+	'tabBarAutoHide': False,
 	'tabInsertsSpaces': True,
 	'tabWidth': 4,
 	'uiLanguage': QLocale.system().name(),
 	'useFakeVim': False,
+	'useWebEngine': False,
 	'useWebKit': False,
 	'windowGeometry': QByteArray(),
 }
