@@ -1,5 +1,7 @@
+# vim: ts=8:sts=8:sw=8:noexpandtab
+#
 # This file is part of ReText
-# Copyright: 2012-2015 Dmitry Shachnev
+# Copyright: 2012-2017 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,12 +16,22 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from markups import MarkdownMarkup
+import os
+import re
+import weakref
+
+from markups import MarkdownMarkup, ReStructuredTextMarkup, TextileMarkup
 from ReText import globalSettings, tablemode, readFromSettings
 
-from PyQt5.QtCore import QSize, Qt
-from PyQt5.QtGui import QColor, QKeyEvent, QPainter, QPalette, QTextCursor, QTextFormat
-from PyQt5.QtWidgets import QLabel, QTextEdit, QWidget
+from PyQt5.QtCore import pyqtSignal, QFileInfo, QRect, QSize, Qt
+from PyQt5.QtGui import QColor, QImage, QKeyEvent, QMouseEvent, QPainter, \
+QPalette, QTextCursor, QTextFormat, QWheelEvent
+from PyQt5.QtWidgets import QFileDialog, QLabel, QTextEdit, QWidget
+
+try:
+	from ReText.fakevimeditor import ReTextFakeVimHandler
+except ImportError:
+	ReTextFakeVimHandler = None
 
 colors = {
 	'marginLine': QColor(0xdc, 0xd2, 0xdc),
@@ -78,9 +90,12 @@ def documentIndentLess(document, cursor, globalSettings=globalSettings):
 	cursor.endEditBlock()
 
 class ReTextEdit(QTextEdit):
+	resized = pyqtSignal(QRect)
+	scrollLimitReached = pyqtSignal(QWheelEvent)
+
 	def __init__(self, parent):
 		QTextEdit.__init__(self)
-		self.tab = parent
+		self.tab = weakref.proxy(parent)
 		self.parent = parent.p
 		self.undoRedoActive = False
 		self.tableModeEnabled = False
@@ -91,6 +106,8 @@ class ReTextEdit(QTextEdit):
 		self.document().blockCountChanged.connect(self.updateLineNumberAreaWidth)
 		self.cursorPositionChanged.connect(self.highlightCurrentLine)
 		self.document().contentsChange.connect(self.contentsChange)
+		if globalSettings.useFakeVim:
+			self.installFakeVimHandler()
 
 	def updateFont(self):
 		self.setFont(globalSettings.editorFont)
@@ -111,29 +128,21 @@ class ReTextEdit(QTextEdit):
 		painter.drawLine(self.marginx, y1, self.marginx, y2)
 		QTextEdit.paintEvent(self, event)
 
+	def wheelEvent(self, event):
+		QTextEdit.wheelEvent(self, event)
+
+		if event.angleDelta().y() < 0:
+			scrollBarLimit = self.verticalScrollBar().maximum()
+		else:
+			scrollBarLimit = self.verticalScrollBar().minimum()
+
+		if self.verticalScrollBar().value() == scrollBarLimit:
+			self.scrollLimitReached.emit(event)
+
 	def scrollContentsBy(self, dx, dy):
 		QTextEdit.scrollContentsBy(self, dx, dy)
 		self.lineNumberArea.update()
 
-	def lineNumberAreaPaintEvent(self, event):
-		painter = QPainter(self.lineNumberArea)
-		painter.fillRect(event.rect(), colorValues['lineNumberArea'])
-		cursor = QTextCursor(self.document())
-		cursor.movePosition(QTextCursor.Start)
-		atEnd = False
-		while not atEnd:
-			rect = self.cursorRect(cursor)
-			block = cursor.block()
-			if block.isVisible():
-				number = str(cursor.blockNumber() + 1)
-				painter.setPen(colorValues['lineNumberAreaText'])
-				painter.drawText(0, rect.top(), self.lineNumberArea.width()-2,
-					self.fontMetrics().height(), Qt.AlignRight, number)
-			cursor.movePosition(QTextCursor.EndOfBlock)
-			atEnd = cursor.atEnd()
-			if not atEnd:
-				cursor.movePosition(QTextCursor.NextBlock)
-
 	def contextMenuEvent(self, event):
 		text = self.toPlainText()
 		dictionary = self.tab.highlighter.dictionary
@@ -142,7 +151,8 @@ class ReTextEdit(QTextEdit):
 		oldcursor = self.textCursor()
 		cursor = self.cursorForPosition(event.pos())
 		pos = cursor.positionInBlock()
-		if pos == len(text): pos -= 1
+		if pos == len(text):
+			pos -= 1
 		curchar = text[pos]
 		isalpha = curchar.isalpha()
 		cursor.select(QTextCursor.WordUnderCursor)
@@ -168,8 +178,6 @@ class ReTextEdit(QTextEdit):
 	def keyPressEvent(self, event):
 		key = event.key()
 		cursor = self.textCursor()
-		if event.text() and self.tableModeEnabled:
-			cursor.beginEditBlock()
 		if key == Qt.Key_Backspace and event.modifiers() & Qt.GroupSwitchModifier:
 			# Workaround for https://bugreports.qt.io/browse/QTBUG-49771
 			event = QKeyEvent(event.type(), event.key(),
@@ -178,20 +186,26 @@ class ReTextEdit(QTextEdit):
 			documentIndentMore(self.document(), cursor)
 		elif key == Qt.Key_Backtab:
 			documentIndentLess(self.document(), cursor)
-		elif key == Qt.Key_Return and not cursor.hasSelection():
-			if event.modifiers() & Qt.ShiftModifier:
-				# Insert Markdown-style line break
-				markupClass = self.tab.getMarkupClass()
-				if markupClass and markupClass == MarkdownMarkup:
-					cursor.insertText('  ')
+		elif key == Qt.Key_Return:
+			markupClass = self.tab.getActiveMarkupClass()
 			if event.modifiers() & Qt.ControlModifier:
 				cursor.insertText('\n')
+				self.ensureCursorVisible()
+			elif self.tableModeEnabled and tablemode.handleReturn(cursor, markupClass,
+					newRow=(event.modifiers() & Qt.ShiftModifier)):
+				self.setTextCursor(cursor)
+				self.ensureCursorVisible()
 			else:
+				if event.modifiers() & Qt.ShiftModifier and markupClass == MarkdownMarkup:
+					# Insert Markdown-style line break
+					cursor.insertText('  ')
 				self.handleReturn(cursor)
 		else:
+			if event.text() and self.tableModeEnabled:
+				cursor.beginEditBlock()
 			QTextEdit.keyPressEvent(self, event)
-		if event.text() and self.tableModeEnabled:
-			cursor.endEditBlock()
+			if event.text() and self.tableModeEnabled:
+				cursor.endEditBlock()
 
 	def handleReturn(self, cursor):
 		# Select text between the cursor and the line start
@@ -202,9 +216,11 @@ class ReTextEdit(QTextEdit):
 		while pos < length and (text[pos] in (' ', '\t')
 		  or text[pos:pos+2] in ('* ', '- ')):
 			pos += 1
+		if pos == length:
+			cursor.removeSelectedText()
 		# Reset the cursor
 		cursor = self.textCursor()
-		cursor.insertText('\n'+text[:pos])
+		cursor.insertText(('\n' + text[:pos]) if pos < length else '\n')
 		self.ensureCursorVisible()
 
 	def lineNumberAreaWidth(self):
@@ -216,12 +232,12 @@ class ReTextEdit(QTextEdit):
 		return 5 + self.fontMetrics().width('9') * digits
 
 	def updateLineNumberAreaWidth(self, blockcount=0):
-		self.lineNumberArea.update()
 		self.setViewportMargins(self.lineNumberAreaWidth(), 0, 0, 0)
 
 	def resizeEvent(self, event):
 		QTextEdit.resizeEvent(self, event)
 		rect = self.contentsRect()
+		self.resized.emit(rect)
 		self.lineNumberArea.setGeometry(rect.left(), rect.top(),
 			self.lineNumberAreaWidth(), rect.height())
 		self.infoArea.updateTextAndGeometry()
@@ -249,11 +265,74 @@ class ReTextEdit(QTextEdit):
 
 	def contentsChange(self, pos, removed, added):
 		if self.tableModeEnabled:
-			markupClass = self.tab.getMarkupClass()
+			markupClass = self.tab.getActiveMarkupClass()
 
 			cursorPosition = self.backupCursorPositionOnLine()
 			tablemode.adjustTableToChanges(self.document(), pos, added - removed, markupClass)
 			self.restoreCursorPositionOnLine(cursorPosition)
+		self.lineNumberArea.update()
+
+	def canInsertFromMimeData(self, mimeData):
+		return mimeData.hasText() or mimeData.hasImage()
+
+	def findNextImageName(self, filenames):
+		highestNumber = 0
+		for filename in filenames:
+			m = re.match(r'image(\d+).png', filename, re.IGNORECASE)
+			if m:
+				number = int(m.group(1))
+				highestNumber = max(number, highestNumber)
+		return 'image%04d.png' % (highestNumber + 1)
+
+	def getImageFilenameAndLink(self):
+		if self.tab.fileName:
+			saveDir = os.path.dirname(self.tab.fileName)
+		else:
+			saveDir = os.getcwd()
+
+		imageFileName = self.findNextImageName(os.listdir(saveDir))
+
+		chosenFileName = QFileDialog.getSaveFileName(self,
+		                                   self.tr('Save image'),
+		                                   os.path.join(saveDir, imageFileName),
+		                                   self.tr('Images (*.png *.jpg)'))[0]
+
+		if chosenFileName:
+			# Use relative links for named documents
+			if self.tab.fileName:
+				link = os.path.relpath(chosenFileName, saveDir)
+			else:
+				link = chosenFileName
+		else:
+			link = None
+
+		return chosenFileName, link
+
+	def insertFromMimeData(self, mimeData):
+		if mimeData.hasImage():
+			fileName, link = self.getImageFilenameAndLink()
+			if fileName:
+				image = QImage(mimeData.imageData())
+				image.save(fileName)
+
+				markupClass = self.tab.getActiveMarkupClass()
+				if markupClass == MarkdownMarkup:
+					imageText = '![%s](%s)' % (QFileInfo(link).baseName(), link)
+				elif markupClass == ReStructuredTextMarkup:
+					imageText = '.. image:: %s' % link
+				elif markupClass == TextileMarkup:
+					imageText = '!%s!' % link
+
+				self.textCursor().insertText(imageText)
+		else:
+			QTextEdit.insertFromMimeData(self, mimeData)
+
+	def installFakeVimHandler(self):
+		if ReTextFakeVimHandler:
+			fakeVimEditor = ReTextFakeVimHandler(self, self.parent)
+			fakeVimEditor.setSaveAction(self.parent.actionSave)
+			fakeVimEditor.setQuitAction(self.parent.actionQuit)
+			self.parent.actionFakeVimMode.triggered.connect(fakeVimEditor.remove)
 
 class LineNumberArea(QWidget):
 	def __init__(self, editor):
@@ -264,8 +343,25 @@ class LineNumberArea(QWidget):
 		return QSize(self.editor.lineNumberAreaWidth(), 0)
 
 	def paintEvent(self, event):
-		if globalSettings.lineNumbersEnabled:
-			return self.editor.lineNumberAreaPaintEvent(event)
+		if not globalSettings.lineNumbersEnabled:
+			return QWidget.paintEvent(self, event)
+		painter = QPainter(self)
+		painter.fillRect(event.rect(), colorValues['lineNumberArea'])
+		cursor = QTextCursor(self.editor.document())
+		cursor.movePosition(QTextCursor.Start)
+		atEnd = False
+		while not atEnd:
+			rect = self.editor.cursorRect(cursor)
+			block = cursor.block()
+			if block.isVisible():
+				number = str(cursor.blockNumber() + 1)
+				painter.setPen(colorValues['lineNumberAreaText'])
+				painter.drawText(0, rect.top(), self.width() - 2,
+					self.fontMetrics().height(), Qt.AlignRight, number)
+			cursor.movePosition(QTextCursor.EndOfBlock)
+			atEnd = cursor.atEnd()
+			if not atEnd:
+				cursor.movePosition(QTextCursor.NextBlock)
 
 class InfoArea(QLabel):
 	def __init__(self, editor):
@@ -277,6 +373,7 @@ class InfoArea(QLabel):
 		palette = self.palette()
 		palette.setColor(QPalette.Window, colorValues['infoArea'])
 		self.setPalette(palette)
+		self.setCursor(Qt.IBeamCursor)
 
 	def updateTextAndGeometry(self):
 		text = self.getText()
@@ -296,3 +393,32 @@ class InfoArea(QLabel):
 		block = cursor.blockNumber() + 1
 		position = cursor.positionInBlock()
 		return template % (block, position)
+
+	def enterEvent(self, event):
+		palette = self.palette()
+		windowColor = QColor(colorValues['infoArea'])
+		windowColor.setAlpha(0x20)
+		palette.setColor(QPalette.Window, windowColor)
+		textColor = palette.color(QPalette.WindowText)
+		textColor.setAlpha(0x20)
+		palette.setColor(QPalette.WindowText, textColor)
+		self.setPalette(palette)
+
+	def leaveEvent(self, event):
+		palette = self.palette()
+		palette.setColor(QPalette.Window, colorValues['infoArea'])
+		palette.setColor(QPalette.WindowText,
+			self.editor.palette().color(QPalette.WindowText))
+		self.setPalette(palette)
+
+	def mousePressEvent(self, event):
+		pos = self.mapToParent(event.pos())
+		pos.setX(pos.x() - self.editor.lineNumberAreaWidth())
+		newEvent = QMouseEvent(event.type(), pos,
+		                       event.button(), event.buttons(),
+		                       event.modifiers())
+		self.editor.mousePressEvent(newEvent)
+
+	mouseReleaseEvent = mousePressEvent
+	mouseDoubleClickEvent = mousePressEvent
+	mouseMoveEvent = mousePressEvent
